import cv2

from pypot.creatures import PoppyErgoJr

if __name__ == '__main__':
    jr = PoppyErgoJr()

    while True:
        img = jr.camera.frame

        for m in jr.marker_detector.markers:
            m.draw_contour(img)

        cv2.imshow('live', img)
        cv2.waitKey(100)



#!/usr/bin/env python

import cv2
import sys
import time

from contextlib import closing
from collections import deque

from pypot.creatures import PoppyErgoJr
from pypot.primitive import LoopPrimitive
from poppy_ergo_jr.postures import IdleBreathing


RETRY = 5.0


def make_jr_or_die_trying():
    while True:
        try:
            jr = PoppyErgoJr()
            break
        except (IndexError, ValueError, OSError):
            print('Tries to connect to the ErgoJr failed!'
                  ' Retry in {}s...'.format(RETRY))
            time.sleep(RETRY)
        # except (EnvironmentError):
            # print('You most likely have to unplug/replug your ErgoJr!')
            # sys.exit(1)

    print('Sucessfully connected to the ErgoJr!')
    return jr


class DemoMode(LoopPrimitive):
    def setup(self):
        self.dance_marker = 233511930
        self.state = 'breathing'
        self._seen_faces = deque([], 30)

        self.breathing = IdleBreathing(self.robot, 1.)
        self.breathing.start()

    def update(self):
        self._seen_faces.append(len(self.robot.face_detector.faces) > 0)

        if self.state == 'breathing':
            marker = [m for m in self.robot.marker_detector.markers
                      if m.id == self.dance_marker]

            if marker:
                self.breathing.stop()

                self.state = 'dancing'
                self.robot.dance.start()

            if sum(self._seen_faces) > .75 * self._seen_faces.maxlen:
                self.state = 'tracking'
                self.breathing.stop()
                self.robot.face_tracking.start()

        elif self.state == 'dancing':
            if self.robot.dance.elapsed_time > 10:
                self.robot.dance.stop()

                self.state = 'breathing'
                self._seen_faces.clear()
                self.breathing.start()

        elif self.state == 'tracking':
            if sum(self._seen_faces) < .25 * self._seen_faces.maxlen:
                self.robot.face_tracking.stop()
                self._seen_faces.clear()

                self.state = 'breathing'
                self.breathing.start()

        img = self.robot.camera.frame
        for face in self.robot.face_detector.faces:
            face.draw(img)
        cv2.imshow('live', img)
        cv2.waitKey(10)

    def teardown(self):
        self.breathing.stop()


if __name__ == '__main__':
    with closing(make_jr_or_die_trying()) as jr:
        demo = DemoMode(jr, 5.)

        print('Slowly raising...')
        jr.safe_power_up.start()
        jr.safe_power_up.wait_to_stop()

        print('Starting the demo mode!')
        demo.start()

        try:
            while True:
                time.sleep(1000)
        except KeyboardInterrupt:
            print('Stopping demo and leaving.')
            demo.stop()



from pypot.primitive import LoopPrimitive


class HeadFollow(LoopPrimitive):
    def __init__(self, robot, freq, detector, tracked_id):
        LoopPrimitive.__init__(self, robot, freq)

        self.dx, self.dy = 60, 50

        def get_marker_pos():
            marker = [m.position for m in getattr(detector, 'markers')
                      if m.id == tracked_id]

            if len(marker) == 1:
                return marker[0]

            return None

        self.get_marker_pos = get_marker_pos

    def setup(self):
        self.robot.rest_posture.start()
        self.robot.rest_posture.wait_to_stop()

        self.rest_pos = {m.name: m.present_position for m in self.robot.motors}
        for m in [self.robot.m1, self.robot.m2]:
            m.moving_speed = 50.

    def update(self):
        c = self.get_marker_pos()

        if c is not None:
            x, y = c

            self.robot.m1.goal_position = self.rest_pos['m1'] - x * self.dx
            self.robot.m2.goal_position = self.rest_pos['m2'] + y * self.dy

    def teardown(self):
        self.robot.rest_posture.start()
        self.robot.rest_posture.wait_to_stop()



from pypot.primitive import Primitive, LoopPrimitive
from pypot.primitive.utils import SimplePosture, Sinus


class BasePosture(SimplePosture):
    @property
    def target_position(self):
        return {m.name: 0. for m in self.robot.motors}


class RestPosture(SimplePosture):
    @property
    def leds(self):
        return {m: 'blue' for m in self.robot.motors}

    @property
    def target_position(self):
        return {
            'm1': 0.,
            'm2': -90.,
            'm3': 35.,
            'm4': 0.,
            'm5': 55.,
            'm6': -5.,
        }

class CuriousPosture(SimplePosture):
    @property
    def target_position(self):
        return {
            'm1': 0.,
            'm2': -15.,
            'm3': 40.,
            'm4': 0.,
            'm5': -35.,
            'm6': -60.
        }

    @property
    def leds(self):
        return {m: 'pink' for m in self.robot.motors}

class TetrisPosture(SimplePosture):
    @property
    def target_position(self):
        return {
            'm1': 0.,
            'm2': -90.,
            'm3': 90.,
            'm4': 0.,
            'm5': -90.,
            'm6': -90.
        }

    @property
    def leds(self):
        return {m: 'yellow' for m in self.robot.motors}

class IdlePosture(SimplePosture):
    @property
    def target_position(self):
        return {
            'm1': 0.0,
            'm2': -110.,
            'm3': 65.0,
            'm4': 0.0,
            'm5': 34.0,
            'm6': 20.0,
        }

    @property
    def leds(self):
        return {m: 'blue' for m in self.robot.motors}

class IdleBreathing(LoopPrimitive):
    def setup(self):
        self.idle = IdlePosture(self.robot, 1.0)
        self.idle.start()
        self.idle.wait_to_stop()

        self.sinus = [
            Sinus(self.robot, 10., [self.robot.m3, ], amp=5, freq=.1,
                  offset=self.robot.m3.present_position),
            Sinus(self.robot, 10., [self.robot.m5], amp=10, freq=.1,
                  offset=self.robot.m5.present_position, phase=3.14),
        ]
        import time
        time.sleep(2)
        [s.start() for s in self.sinus]

        for m in self.robot.motors:
            m.led = 'blue'

    def update(self):
        pass

    def teardown(self):
        for m in self.robot.motors:
            m.led = 'off'
        [s.stop() for s in self.sinus]

        self.idle.start()
        self.idle.wait_to_stop()


class SafePowerUp(Primitive):
    def run(self):
        idle_prim = IdlePosture(self.robot, 3.0)
        idle_prim.start()
        idle_prim.wait_to_stop()

    def teardown(self):
        for m in self.robot.motors:
            m.moving_speed = 0.



from numpy import sum
from functools import partial

from pypot.creatures import AbstractPoppyCreature
from pypot.creatures.ik import IKChain

from .primitives.dance import Dance
from .primitives.face_tracking import FaceTracking
from .primitives.tracking_feedback import TrackingFeedback
from .primitives.postures import (BasePosture, RestPosture,
                                  CuriousPosture, TetrisPosture,
                                  SafePowerUp)


class RoboticiaUno(AbstractPoppyCreature):
    @classmethod
    def setup(cls, robot):
        robot._primitive_manager._filter = partial(sum, axis=0)

        c = IKChain.from_poppy_creature(robot,
                                        motors=robot.motors,
                                        passiv=[],
                                        tip=[0, 0, -0.07])

        robot.chain = c

        robot.attach_primitive(SafePowerUp(robot), 'safe_power_up')

        robot.attach_primitive(Dance(robot), 'dance')

        robot.attach_primitive(BasePosture(robot, 2.), 'base_posture')
        robot.attach_primitive(RestPosture(robot, 2.), 'rest_posture')
        robot.attach_primitive(CuriousPosture(robot, 2.), 'curious_posture')
        robot.attach_primitive(TetrisPosture(robot, 2.), 'tetris_posture')

        if not robot.simulated and hasattr(robot, 'marker_detector'):
            robot.attach_primitive(TrackingFeedback(robot, 25.),
                                   'tracking_feedback')

        for m in robot.motors:
            m.pid = (4, 2, 0)
            m.torque_limit = 70.
            m.led = 'off'

        if not robot.simulated and hasattr(robot, 'face_tracking'):
            robot.attach_primitive(FaceTracking(robot, 10,
                                                robot.face_detector),
                                   'face_tracking')



from pypot.primitive import LoopPrimitive


class FaceTracking(LoopPrimitive):
    def __init__(self, robot, freq, face_detector):
        LoopPrimitive.__init__(self, robot, freq)

        self.face_detector = face_detector

        self.dx, self.dy = 60, 50
        self._tracked_face = None

    def setup(self):
        self.robot.rest_posture.start()
        self.robot.rest_posture.wait_to_stop()

        for m in self.robot._robot.motors:
            m.led = 'yellow'

        self.rest_pos = {m.name: m.present_position for m in self.robot.motors}
        for m in [self.robot.m1, self.robot.m5]:
            m.moving_speed = 50.

        # TODO: That's a really nasty way to circumvent prim sandboxing
        # How should we do that in a more elegant way?
        img = getattr(self.face_detector._robot,
                      self.face_detector._names[0]).frame
        self.img_size = tuple(reversed(img.shape[:2]))

    def update(self):
        faces = self.face_detector.faces

        # use filter to keep only closest faces to preivoulsy tracked one

        if faces:
            x, y = faces[0].center
            x = (float(x) / self.img_size[0]) * 2 - 1
            y = (float(y) / self.img_size[1]) * 2 - 1

            self.robot.m1.goal_position = self.rest_pos['m1'] -x * self.dx
            self.robot.m5.goal_position = self.rest_pos['m5'] + y * self.dy

    def teardown(self):
        for m in self.robot._robot.motors:
            m.led = 'off'



import cv2
import time
import pylab
import psutil

from contextlib import closing
from numpy import mean, std

from pypot.primitive.utils import PositionWatcher
from pypot.creatures import PoppyErgoJr
from hampy import detect_markers

D = 10

cpu = []

if __name__ == '__main__':
    psutil.cpu_percent()
    time.sleep(.5)

    with closing(PoppyErgoJr()) as jr:
        jr.rest_posture.start()
        jr.rest_posture.wait_to_stop()

        traj_rec = PositionWatcher(jr, 25., jr.motors)

        jr.dance.start()
        traj_rec.start()

        t0 = time.time()
        while time.time() - t0 < D:
            cpu.append(psutil.cpu_percent())

            img = jr.camera.frame
            markers = detect_markers(img)
            for m in markers:
                m.draw_contour(img)

            time.sleep(.1)

        jr.dance.stop()
        traj_rec.stop()

        print('CPU M={}% (STD={})'.format(mean(cpu) * 4, std(cpu) * 4))

        fig = pylab.figure()
        ax = pylab.axes()
        traj_rec.plot(ax)
        pylab.show()
        fig.savefig('bench.png')



import time

from pypot.primitive import Primitive


class Jump(Primitive):
    def setup(self):
        self.up = {'m1': 0, 'm2': -10, 'm3': -20, 'm4': 0, 'm5': -35, 'm6': -35}
        self.down = {'m1': 0, 'm2': -75, 'm3': 55, 'm4': 0, 'm5': 35, 'm6': 35}

        for m in self.robot.motors:
            m.compliant = False
            m.pid = (10., 0., 0.)
            m.moving_speed = 0.

    def run(self):
        while not self.should_stop():
            if self.should_pause():
                self.wait_to_resume()

            for name, p in self.up.items():
                m = getattr(self.robot, name)
                m.moving_speed = 0
                m.goal_position = p
            time.sleep(.2)

            for name, p in self.down.items():
                m = getattr(self.robot, name)
                m.moving_speed = 250.
                m.goal_position = p
            time.sleep(.4)

    def teardown(self):
        for m in self.robot.motors:
            m.pid = (4., 0., 0.)



from collections import deque

from pypot.primitive import LoopPrimitive


class TrackingFeedback(LoopPrimitive):
    def setup(self):
        self._old = False

    def update(self):
        if not hasattr(self, 'q'):
            self.q = deque([], 6)

        self.q.append(len(self.robot.marker_detector.markers) > 0)
        tracking = any(self.q)

        if tracking != self._old:
            for m in self.robot.tip:
                self.affect_once(m, 'led', 'green' if tracking else 'off')

            self._old = tracking

    def teardown(self):
        for m in self.robot.motors:
            m.led = 'off'



from pypot.primitive import LoopPrimitive
from pypot.primitive.utils import Sinus


class Dance(LoopPrimitive):
    def __init__(self, robot):
        LoopPrimitive.__init__(self, robot, 1.)

    def setup(self):
        for m in self.robot.motors:
            m.compliant = False

        self.sinus = [
            Sinus(self.robot, 25., [self.robot.m1], amp=90., freq=0.25),
            Sinus(self.robot, 25., [self.robot.m4], amp=90., freq=0.25, phase=180.),

            Sinus(self.robot, 25., [self.robot.m5], amp=30, freq=.8),
            Sinus(self.robot, 25., [self.robot.m6], amp=30, freq=.8, phase=180),

            Sinus(self.robot, 25., self.robot.motors, amp=10, freq=.1)
        ]

        init_pos = dict([(m.name, 0.0) for m in self.robot.motors])
        self.robot.goto_position(init_pos, 3., wait=True)

        for m in self.robot.motors:
            m.moving_speed = 150.

        for m in self.robot.motors:
            m.led = 'green'

        [s.start() for s in self.sinus]

    def update(self):
        pass

    def teardown(self):
        [s.stop() for s in self.sinus]

        for m in self.robot.motors:
            m.led = 'off'
